@@ -12,20 +12,26 @@ class TestEndtoEnd:
     def test_e2e(self):
         load_dotenv()  # take environment variables from .env.
         API_KEY = os.environ.get("LOTUS_API_KEY")
+        host = os.environ.get("LOTUS_HOST", "https://api.uselotus.io")
         lotus.api_key = API_KEY
-        lotus.strict=True
+        lotus.strict = True
+        lotus.host = host
 
-        
-        plan_id = "plan_aead7e8eb07249c2b2610e936d24a356"
+        now = datetime.datetime.now(datetime.timezone.utc)
+        now_minus_day = now - relativedelta.relativedelta(days=1)
         id = uuid.uuid4().hex
+
+        plan_id = os.environ.get("PLAN_ID", "plan_aead7e8eb07249c2b2610e936d24a356")
+        addon_id = os.environ.get("ADDON_ID", "addon_4f236c4f262443179a0d99f15405e7de")
+        ## CREATE CUSTOMER
         response = lotus.create_customer(
             customer_id=id,
             customer_name="Test Customer",
             email=f"{id}@email.com",
         )
         assert response["customer_id"] == id
-        now = datetime.datetime.now(datetime.timezone.utc)
-        now_minus_day = now - relativedelta.relativedelta(days=1)
+
+        ## LIST + CREATE CREDIT
         credits = lotus.list_credits(
             customer_id=id,
         )
@@ -34,7 +40,7 @@ class TestEndtoEnd:
             customer_id=id,
             amount=100,
             currency_code="USD",
-            expires_at=now+relativedelta.relativedelta(days=7),
+            expires_at=now + relativedelta.relativedelta(days=7),
             description="Test Credit",
         )
         credits = lotus.list_credits(
@@ -49,6 +55,8 @@ class TestEndtoEnd:
         )
         assert len(credits) == 1
         credit = credits[0]
+
+        ## UPDATE + VOID CREDIT
         lotus.update_credit(
             credit_id=credit["credit_id"],
             expires_at=now + relativedelta.relativedelta(days=5),
@@ -63,6 +71,8 @@ class TestEndtoEnd:
         assert len(credits) == 1
         credit = credits[0]
         assert len(credit["drawdowns"]) > 0
+
+        ## CREATE SUBSCRIPTION
         response = lotus.create_subscription(
             customer_id=id,
             plan_id=plan_id,
@@ -72,6 +82,23 @@ class TestEndtoEnd:
         assert response["start_date"] == now_minus_day
         assert response["customer"]["customer_id"] == id
         assert response["billing_plan"]["plan_id"] == plan_id
+
+        ## ATTACH ADDON
+        response = lotus.attach_addon(
+            attach_to_customer_id=id,
+            addon_id=addon_id,
+            attach_to_subscription_filters=[{"property_name": "region", "value": "US"}],
+            attach_to_plan_id=plan_id,
+            quantity=10,
+        )
+        now = datetime.datetime.now(datetime.timezone.utc)
+        assert response["addon"]["addon_id"] == addon_id
+        assert now_minus_day < response["start_date"] < now
+        assert response["parent"]["plan_detail"]["plan_id"] == plan_id
+        assert response["fully_billed"] is True
+        assert response["auto_renew"] is False
+
+        ## TRIACK EVENT
         lotus.track_event(
             customer_id=id,
             event_name="test_event",
@@ -87,6 +114,8 @@ class TestEndtoEnd:
             event_name="test_event",
             properties={"region": "US", "mb_used": 150},
         )
+
+        ## LIST CUSTOMER + SUBSCRIPTIONS
         customers = lotus.list_customers()
         assert len(customers) > 0
         cust_subs = lotus.list_subscriptions(customer_id=id)
@@ -95,6 +124,8 @@ class TestEndtoEnd:
         assert cust_subs[0]["subscription_filters"][0]["property_name"] == "region"
         assert cust_subs[0]["subscription_filters"][0]["value"] == "US"
         assert cust_subs[0]["auto_renew"] is True
+
+        ## UPDATE SUBSCRIPTION
         sub = lotus.update_subscription(
             customer_id=id,
             plan_id=plan_id,
@@ -102,29 +133,45 @@ class TestEndtoEnd:
         )
         assert len(sub) == 1
         assert sub[0]["auto_renew"] is False
+
+        ## CANCEL ADDON
+        response = lotus.cancel_addon(
+            attached_customer_id=id,
+            addon_id=addon_id,
+            attached_plan_id=plan_id,
+            flat_fee_behavior="charge_prorated",
+        )[0]
+        now = datetime.datetime.now(datetime.timezone.utc)
+        assert response["end_date"] < now
+        assert response["fully_billed"] is True
+
+        ## METRIC ACCESS
         access = lotus.check_metric_access(
             customer_id=id,
             metric_id="metric_a25d887196464a1389fd65194f9e1d7f",
             subscription_filters=[{"property_name": "region", "value": "US"}],
         )
-
         assert access["metric"]["event_name"] == "test_event"
         assert access["access_per_subscription"][0]["metric_usage"] >= 0
         assert access["access_per_subscription"][0]["metric_total_limit"] is None
-        access = lotus.check_metric_access( #bogus metric id
+        access = lotus.check_metric_access(  # bogus metric id
             customer_id=id,
             metric_id="metric_5da8be769cdf4e3fa8233a22fb920733",
             subscription_filters=[{"property_name": "region", "value": "US"}],
         )
         assert access["access_per_subscription"][0]["metric_usage"] == 0
         assert access["access_per_subscription"][0]["metric_total_limit"] == 0
+
+        ## FEATURE ACCESS
         feature_access = lotus.check_feature_access(
             customer_id=id, feature_id="feature_6b037a7bbce44ee98a65a04e97e2f5dd"
         )
         assert feature_access["feature"]["feature_name"] == "test_feature"
         assert feature_access["access"] is True
         customer = lotus.get_customer(customer_id=id)
-        assert customer["total_amount_due"] == 50
+        assert customer["total_amount_due"] == 550  # 50*10 (ADDON) + 50
+
+        ## CANCEL SUBSCRIPTION
         canceled_sub = lotus.cancel_subscription(
             customer_id=id, invoicing_behavior="invoice_now"
         )
@@ -133,6 +180,7 @@ class TestEndtoEnd:
         now = datetime.datetime.now(datetime.timezone.utc)
         assert canceled_sub["end_date"] <= now
         assert canceled_sub["fully_billed"] is True
+
+        ## LIST PLANS
         plans = lotus.list_plans()
         assert len(plans) > 0
-        assert len(plans) > 0
